/* 
 * Copyright (c)2007, Dustin Spicuzza
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * The name of the Dustin Spicuzza may not be used to endorse or promote 
 *       products derived from this software without specific prior written 
 *       permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Dustin Spicuzza ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Dustin Spicuzza BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using CyUSB;
using CyUSB.Async;

using System.Threading;

namespace USBTest {


	public partial class Form1 : System.Windows.Forms.Form {


		USBDeviceList usbDevices;
		CyUSBDevice usbDevice;

		CyUSBAsyncWrapper usbComm;


		public Form1(){

			InitializeComponent();

			App_PnP_Callback evHandler = new App_PnP_Callback(PnP_Event_Handler);
			usbDevices = new USBDeviceList(0xff, evHandler);
            usbConfig1.Init(usbDevices);
		}


		public void PnP_Event_Handler(IntPtr pnpEvent, IntPtr hRemovedDevice) 
        {

			if (pnpEvent.Equals(CyConst.DBT_DEVICEREMOVECOMPLETE)) 
            {
                usbDevices.Remove(hRemovedDevice);
                // Other removal event handling
			}

			if (pnpEvent.Equals(CyConst.DBT_DEVICEARRIVAL)) 
            {
                usbDevices.Add();
                // Other arrival event handling  
			}

			usbConfig1.UsbConfig_ReConfig();
		}



		private void button2_Click(object sender, EventArgs e) {
			groupBox1.Enabled = !groupBox1.Enabled;
			usbConfig1.Enabled = !groupBox1.Enabled;

			if ( groupBox1.Enabled ) 
            {
				button2.Text = "Deselect";
				ConfigureDevice();
			} else
				button2.Text = "Select";

		}

		/// <summary>
		/// Configures the selected device
		/// </summary>
		private void ConfigureDevice() 
        {

			if ( usbConfig1.comboBox1.SelectedIndex == -1 )
				return;

			// unconfigure previous device (abort its receive thread)

			// configure new device

			usbDevice = usbDevices[usbConfig1.comboBox1.SelectedIndex] as CyUSBDevice;

			// transfer data from pc to the usb device
			usbComm = new CyUSBAsyncWrapper(usbDevice.BulkOutEndPt);

			textBox2.Text = "";
        }

		ASCIIEncoding encoder = new ASCIIEncoding();

		
		// the following routine really sucks... should do bounds checking

		private void button1_Click(object sender, EventArgs e) {

			byte[] data = encoder.GetBytes(textBox1.Text);
			usbComm.BeginTransfer(data, data.Length, new AsyncCallback(EndTransfer), data);

		}

		// called on another thread when the transfer is complete
		void EndTransfer(IAsyncResult result) {
			usbComm.EndTransfer(result);
		}


	}
}